@@ -10,7 +10,7 @@ e.g. an :py:class:`Option<ase2sprkkr.common.options.Option>` or
 ), or write such object to a file.
 """
 
-from ..common.grammar_types  import type_from_type, type_from_value, BaseType
+from ..common.grammar_types  import type_from_type, type_from_value, GrammarType
 from ..common.grammar  import delimitedList, end_of_file, generate_grammar
 import pyparsing as pp
 from ..common.misc import OrderedDict
@@ -20,6 +20,8 @@ import numpy as np
 import inspect
 from .misc import cache
 import itertools
+import builtins
+from typing import Dict
 
 #:This serves just for dealing with various pyparsing versions
 _parse_all_name = 'parse_all' if \
@@ -57,8 +59,10 @@ class BaseDefinition:
    the behavior.
    """
 
-   def __init__(self, name, alternative_names=None, is_optional=False, is_hidden=False,
-                name_in_grammar=None, help=None, description=None, write_alternative_name=False):
+   def __init__(self, name, alternative_names=None,
+                is_optional=False, is_hidden=False,is_expert=False,
+                name_in_grammar=None, info=None, description=None, write_alternative_name=False,
+                ):
        """
        Parameters
        ----------
@@ -68,20 +72,24 @@ class BaseDefinition:
         alternative_names: str or [str]
           Alternative names that can denotes the value
 
+        is_optional: boolean
+          If True, this section/value can be missing in the .pot/task file
+
         is_hidden: boolean
           Hidden values are not offered to a user, usually they are
           set by another object (and so a direct setting of their values
           has no sense)
 
+        is_expert: boolean
+          Expert values/sections are not required and they are somewhat hidden
+          from the user
+
         name_in_grammar: boolean or None
           If False, there the name of the variable is not printed in the
           configuration file. The variable is recognized by its position.
           If None, the default class value is used
 
-        is_optional: boolean
-          If True, this section/value can be missing in the .pot/task file
-
-        help: str
+        info: str
           A short help message for the value/section. It will be the perex for description.
 
         description: str
@@ -97,15 +105,31 @@ class BaseDefinition:
        of the alternative names.
        """
        self.is_optional = is_optional
+       self.is_expert = is_expert
+       self.is_hidden = is_hidden
        """ Is it required part of configuration (or can it be ommited)? """
        self.write_alternative_name = write_alternative_name
        self.name_in_grammar = self.__class__.name_in_grammar \
                                if name_in_grammar is None else name_in_grammar
-       self.help = help
+       self._info = info
        """ A short help text describing the content for the users. """
-       self.description = description
+       self._description = description
        """ A longer help text describing the content for the users. """
 
+   def info(self, generic:bool=True) -> str:
+       """ Return short help string.
+
+       Parameters
+       ----------
+       generic
+         If the definition has no help specified and generic is True, return a (not saying much) generic help string
+       """
+       out = self._info
+       if not out and generic:
+          out = self._generic_info()
+       return out
+
+
    def create_object(self, container=None):
        """ Creates Section/Option/.... object (whose properties I define) """
        return self.result_class(self, container)
@@ -115,6 +139,19 @@ class BaseDefinition:
        with generate_grammar():
          return self._grammar()
 
+   def description(self):
+       out = [
+          self.info(), ''
+       ]
+
+       out.append("Data description\n"
+                  "----------------")
+       out.append(self.data_description(additional = True))
+       if self._description:
+          out.append(' ')
+          out.append(self._description)
+       return '\n'.join(out)
+
    def _tuple_with_my_name(self, expr, delimiter=None, has_value=True):
         """ Create the grammar returning tuple (self.name, <result of the expr>) """
         if self.name_in_grammar:
@@ -143,16 +180,34 @@ class BaseDefinition:
 
    do_not_skip_whitespaces_before_name = False
 
+   def _get_copy_args(self)->Dict[str, str]:
+       """
+       Compute the dictionary that defines the attributes to create a copy of this object.
+
+       Returns
+       -------
+       copy: Dict
+          The returning dictionary has this structure:
+          { name of the argument of the __init__ function : name of the object attribute }
+       """
+
+       if not '_copy_args' in self.__class__.__dict__:
+          args = inspect.getfullargspec(self.__class__.__init__).args[1:]
+          self.__class__._copy_args = { v: '_'+v if '_'+v in self.__dict__ else v for v in args if v not in self._copy_excluded_args }
+       return self.__class__._copy_args
+
+   _copy_excluded_args = ['expert']
+
 class BaseValueDefinition(BaseDefinition):
 
   result_class = Option
 
   name_in_grammar = None
 
-  def __init__(self, name, type, default_value=None, alternative_names=None,
-               fixed_value=None, required=None, help=None, description=None,
-               is_hidden=False, name_in_grammar=None, name_format=None,
-               is_optional=None):
+  def __init__(self, name, type=None, default_value=None, alternative_names=None,
+               fixed_value=None, required=None, info=None, description=None,
+               is_hidden=False, is_optional=None, is_expert=False,
+               name_in_grammar=None, name_format=None, expert=None):
     """
     Creates the object
 
@@ -161,11 +216,14 @@ class BaseValueDefinition(BaseDefinition):
     name: str
       Name of the configuration value
 
-    type: BaseType
-      Configuration value type
+    type: Optional[GrammarType|mixed]
+      Configuration value data type.
+      If it is set to anyting what is not derived from GrammarType, the given value is used as the default value
+      and the data type is derived from it.
+      If it is None, the default value have to be set using ``expert`` parameter.
 
     default: mixed
-      Default value for configuration
+      Default value for the configuration option.
 
     alternative_names: str or [str]
       Value can have an alternative name (that alternativelly denotes the value)
@@ -176,46 +234,72 @@ class BaseValueDefinition(BaseDefinition):
     required: bool
       Required option can not be set to None (however, a required one
       can be still be optional, if it has a default values).
-      If required = None, required = not is_optional and default_value is None
+      If required = None, it is set to True if both the conditions are met
+        * the value is not expert
+        * the optional is not True and the option has not a default_value
+
+    is_optional: bool or None
+      If True, the value can be omited, if the fixed order is required
+      None means True just if required is False (or it is determined to be False),
+      see the ``required`` parameter.
+
+    is_hidden: bool
+      The value is hidden from the user (no container.name access to the value).
+
+    is_expert: Union[bool|mixed]
+      Expert values are somewhat hidden (e.g. listed at end) from the user.
+      Expert values are not exported to the result, if they are set to the
+      default value.
 
     name_in_grammar: bool or None
       The value in the conf file is prefixed by <name><name_value_delimiter>
       If None, the default type value (type.name_in_grammar) is used
 
-    is_optional: bool or None
-      If True, the value can be omited, if the fixed order is required
-      None means True if required is False
-
-    is_hidden: bool
-      The value is hidden from the user (no container.name access to the value)
-
     name_format: str or None
       The way how the name is written
+
+    expert: Optional[mixed]
+      If not None, set ``is_expert`` to True, ``default_value`` to the given value and
+      ``required`` to False. Note, that also ``type`` can be determined from such given
+      ``default_value``.
     """
-    self.type = type_from_type(type)
-    if default_value is None and not isinstance(self.type, BaseType):
+    if expert is not None:
+       if type is None:
+          type = expert
+       else:
+          default_value=expert
+       is_expert = True
+       required = False
+    elif type is None:
+       raise TypeError("The data-type of the configuration value is required.")
+
+    if default_value is None and not isinstance(type, (GrammarType, builtins.type)):
        self.type = type_from_value(type)
        self.default_value = self.type.convert(type)
     else:
+       self.type = type_from_type(type)
        self.default_value = self.type.convert(default_value) if default_value is not None else None
-    assert isinstance(self.type, BaseType), "grammar_type (sprkkr.common.grammar_types.BaseType descendat) required as a value type"
+    assert isinstance(self.type, GrammarType), "grammar_type (sprkkr.common.grammar_types.GrammarType descendat) required as a value type"
 
     if self.default_value is None and self.type.default_value is not None:
        self.default_value = self.type.default_value
 
     if required is None:
-       required = not is_optional and default_value is None
+       required = not is_expert and (not is_optional and default_value is None)
 
     if is_optional is None:
        is_optional = required is False
 
+    configuration_type_name = 'OPTION'
+
     super().__init__(
          name = name,
          alternative_names = alternative_names,
          is_optional = is_optional,
          is_hidden = is_hidden,
+         is_expert = is_expert,
          name_in_grammar = name_in_grammar,
-         help = help,
+         info=info,
          description = description
     )
 
@@ -224,9 +308,6 @@ class BaseValueDefinition(BaseDefinition):
 
     self.fixed_value = self.type.convert(fixed_value) if fixed_value is not None else None
     self.required = self.default_value is not None if required is None else required
-    self.help = None
-    self.is_hidden = is_hidden
-    self.is_optional = is_optional
     self.name_format = name_format
 
   @property
@@ -236,6 +317,40 @@ class BaseValueDefinition(BaseDefinition):
        return "{:{}}".format(name, self.name_format)
     return name
 
+  def data_description(self, prefix='', additional=False):
+    """
+    Return the description of the contained data type and their type.
+    """
+    out = f"{self.name} : {self.type}"
+    value = self.get_value()
+    if value is not None:
+       out+=f" ≝ {value}"
+
+    flags = []
+    if self.is_optional:
+       flags.append('optional')
+    if self.is_hidden:
+       flags.append('hidden')
+    if self.is_hidden:
+       flags.append('expert')
+    if self.fixed_value is not None:
+       flags.append('read_only')
+    if flags:
+       flags = ', '.join(flags)
+       out+= f"  ({flags})"
+    out = prefix + out
+    if additional:
+       add = self.additional_description()
+       if add:
+          out+='\n' + add
+    return out
+
+  def additional_description(self, prefix=''):
+    """ Return the additional runtime-documentation for the configuration value.
+        E.g. return the possible choices for the value, etc...
+    """
+    return self.type.additional_description(prefix)
+
   def validate(self, value):
     if value is None:
        if self.required:
@@ -296,6 +411,7 @@ class BaseValueDefinition(BaseDefinition):
     return out
 
   def get_value(self, option=None):
+     """ Return the default or fixed value of this option. """
      if self.fixed_value is not None:
         return self.fixed_value
      if self.default_value is not None:
@@ -327,11 +443,8 @@ class BaseValueDefinition(BaseDefinition):
         self.type.write(f, value)
      return True
 
-
   def copy(self, **kwargs):
-     if not '_init_args' in self.__class__.__dict__:
-        self.__class__._init_args = inspect.getfullargspec(self.__init__).args[1:]
-     default = { k: getattr(self, k) for k in self._init_args }
+     default = { k: getattr(self, v) for k,v in self._get_copy_args().items() }
      default.update(kwargs)
      return self.__class__(**default)
 
@@ -339,6 +452,9 @@ class BaseValueDefinition(BaseDefinition):
      del self.section[name]
      return self
 
+  def _generic_info(self):
+      return f"Configuration value {self.name}"
+
 def add_excluded_names_condition(element, names):
     """ Add the condition to the element, that
     its value is not any of given names """
@@ -364,14 +480,14 @@ class BaseContainerDefinition(BaseDefinition):
            items[value.name] = value
         return items
 
-    def __init__(self, name, members=[], alternative_names=[], help=None, description=None, is_hidden=False, has_hidden_members=False, is_optional=False, name_in_grammar=None, force_order=None):
+    def __init__(self, name, members=[], alternative_names=[], info=None, description=None, is_hidden=False, has_hidden_members=False, is_optional=False, name_in_grammar=None, force_order=None):
        super().__init__(
            name = name,
            alternative_names = alternative_names,
            is_optional = is_optional,
            is_hidden = is_hidden,
            name_in_grammar = name_in_grammar,
-           help = help,
+           info = info,
            description = description
        )
 
@@ -388,6 +504,78 @@ class BaseContainerDefinition(BaseDefinition):
        if force_order is not None:
           self.force_order = force_order
 
+    configuration_type_name = 'SECTION'
+    """ Name of the container type in the runtime documentation """
+
+    def data_description(self, prefix='', additional:bool=False):
+        """
+        Return the runtime documentation for the configuration described by this object.
+
+        Parameters
+        ----------
+        additional
+          If it is False, only one line with the section name and basic info is returned.
+          Otherwise, the items contained in the section are listed.
+        """
+        def container_name():
+            out = self.configuration_type_name
+            if out:
+               out+=' '
+            out+=self.name
+            return out
+
+        out = f"{prefix}{container_name()}"
+        flags = []
+        if self.force_order:
+           flags.append('order fixed')
+        if self.is_hidden:
+           flags.append('hidden')
+        if self.is_optional:
+           flags.append('optional')
+        if self.is_expert:
+           flags.append('optional')
+        if flags:
+           flags = join(', ').join(flags)
+           out+=f" ({flags})"
+
+        if additional:
+           add = self.additional_description()
+           if add:
+              out+=f' contains:\n{add}'
+        return out
+
+    def additional_description(self, prefix=''):
+        """
+        Return the description (documentation for runtime) of the items in the container.
+        """
+        cprefix=prefix+'    '
+        out = []
+
+        def write(i):
+           s = i.data_description(cprefix)
+           if not '\n' in s:
+              info = i.info(False)
+              if info:
+                 s = s + (' ' * (max(40 - len(s), 0) + 2)) + info
+           out.append(s)
+
+        expert = False
+        for i in self:
+            if not i.is_expert:
+               write(i)
+            else:
+               expert=True
+
+        if expert:
+          out.append(f'\n{prefix}  Expert options')
+          out.append(prefix +   '  --------------')
+
+          for i in self:
+              if i.is_expert:
+                 write(i)
+
+        return '\n'.join(out)
+
     def __iter__(self):
         return iter(self._members.values())
 
@@ -410,8 +598,6 @@ class BaseContainerDefinition(BaseDefinition):
         del self._members[name]
         return self
 
-    _init_args = inspect.getfullargspec(__init__).args[3:]
-
     def copy(self, args=[], items=[], remove=[], defaults={}, **kwargs):
         """ Copy the section with the contained values modified by the arguments."""
         members = OrderedDict( ( (k,i.copy()) for k,i in self._members.items() ) )
@@ -421,9 +607,10 @@ class BaseContainerDefinition(BaseDefinition):
         for i,v in defaults.items():
             members[i].default_value = members[i].type.convert(v)
 
-        default = { k: getattr(self, k) for k in self._init_args }
+        default = { k: getattr(self, v) for k,v in self._get_copy_args().items() }
         default.update(kwargs)
-        return self.__class__(self.name, members=members, **default)
+        default['members'] = members
+        return self.__class__(**default)
 
     def create_object(self, container=None):
         return self.result_class(self, container)
@@ -584,8 +771,6 @@ class BaseContainerDefinition(BaseDefinition):
            result = start + result
         return self.parse(result, whole_string)
 
-
-
 class BaseSectionDefinition(BaseContainerDefinition):
    """ Base class for definition of the sections in Pot or InputParameters files.
 
@@ -612,6 +797,9 @@ class BaseSectionDefinition(BaseContainerDefinition):
            out = out | pp.Empty().setParseAction(lambda x: df)
         return out
 
+   def _generic_info(self):
+      return f"Configuration section {self.name}"
+
 
 class ConfDefinition(BaseContainerDefinition):
    """ From this class, the definition of the format of a whole configuration file should be derived.
