@@ -32,12 +32,19 @@ class Occupation:
       self._normalize()
       self._update_atoms()
 
+  def items(self):
+      """ dict.items() like enumeration """
+      return self._occupation.items()
+
   def __repr__(self):
       return f"Occupation {self._occupation}"
 
   def __str__(self):
       return f"Occupation {self._occupation}"
 
+  def __iter__(self):
+      return iter(self._occupation)
+
   def _update_atoms(self):
       if self._site:
          self._site.update_atoms()
